from django.db import models

try:
    from django.core.urlresolvers import reverse
except ImportError:
    from django.urls import reverse

import json
import os
import sys
from django.core.validators import MinValueValidator
from carrot.exceptions import CarrotConfigException

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.append(BASE_DIR + '/carrot')


class MessageLog(models.Model):
    """
    MessageLogs store information about a carrot task

    Lifecycle:
        #. A :class:`carrot.objects.Message` object is created and published.
        #. The act of publishing the message creates a MessageLog object with the status 'PUBLISHED'. The task now sits
           in the RabbitMQ queue until it has been consumed
        #. When a consumer digests the message, the status is updated to 'COMPLETED' if the task completes successfully
           or 'FAILED' if it encounters an exception. The output, traceback, exception message and logs are written
           back to the MessageLog object
        #. If a task has failed, it can be requeued. Requeueing a task will create a new :class:`carrot.objects.Message`
           object with the same parameters. In this case, the originally MessageLog object will be deleted
        #. If the task has been completed successfully, it will be deleted three days after completion, provided that
           the :function:`carrot.helper_tasks.cleanup` has not been disabled

    """
    STATUS_CHOICES = (
        ('PUBLISHED', 'Published'),
        ('IN_PROGRESS', 'In progress'),
        ('FAILED', 'Failed'),
        ('COMPLETED', 'Completed'),
    ) #:

    status = models.CharField(max_length=11, choices=STATUS_CHOICES, default='PUBLISHED')
    exchange = models.CharField(max_length=200, blank=True, null=True) #: the exchange
    queue = models.CharField(max_length=200, blank=True, null=True)
    routing_key = models.CharField(max_length=200, blank=True, null=True)
    uuid = models.CharField(max_length=200)
    priority = models.PositiveIntegerField(default=0)

    task = models.CharField(max_length=200)#: the import path for the task to be executed
    task_args = models.TextField(null=True, blank=True, verbose_name='Task positional arguments')
    content = models.TextField(null=True, blank=True, verbose_name='Task keyword arguments')

    exception = models.TextField(null=True, blank=True)
    traceback = models.TextField(null=True, blank=True)
    output = models.TextField(null=True, blank=True)

    publish_time = models.DateTimeField(null=True, blank=True)
    failure_time = models.DateTimeField(null=True, blank=True)
    completion_time = models.DateTimeField(null=True, blank=True)

    log = models.TextField(blank=True, null=True)

    @property
    def virtual_host(self):
        from carrot.utilities import get_host_from_name
        try:
            return str(get_host_from_name(self.queue))
        except CarrotConfigException:
            """
            This exception may get raised here when a MessageLog is created with a queue that is later removed from the
            Django config. This method now returns `None` in these cases, so as not to break the monitor   

            Refer to https://github.com/chris104957/django-carrot/issues/81
            """
            pass

    @property
    def keywords(self):
        """
        Used in :class:`carrot.views.MessageView` to display the keyword arguments as a table
        """
        return json.loads(self.content or '{}')

    def __str__(self):
        return self.task

    @property
    def positionals(self):
        import ast
        if self.task_args == '()':
            return ()
        else:
            return [ast.literal_eval(arg.strip()) for arg in self.task_args[1:-1].split(',') if arg != '']

    # noinspection PyTypeChecker
    def requeue(self):
        """
        Sends a failed MessageLog back to the queue. The original MessageLog is deleted
        """
        from carrot.utilities import publish_message
        msg = publish_message(self.task, *self.positionals, priority=self.priority, queue=self.queue,
                              exchange=self.exchange, routing_key=self.routing_key, **self.keywords)

        if self.id:
            self.delete()

        return msg

    class Meta:
        ordering = '-failure_time', '-completion_time', 'status', '-priority', '-publish_time',


class ScheduledTask(models.Model):
    """
    A model for scheduling tasks to run at a certain interval
    """
    INTERVAL_CHOICES = (
        ('seconds', 'seconds'),
        ('minutes', 'minutes'),
        ('hours', 'hours'),
        ('days', 'days'),
    )

    interval_type = models.CharField(max_length=200, choices=INTERVAL_CHOICES, default='seconds')
    interval_count = models.PositiveIntegerField(default=1, validators=[MinValueValidator(1)])

    exchange = models.CharField(max_length=200, blank=True, null=True)
    routing_key = models.CharField(max_length=200, blank=True, null=True)
    queue = models.CharField(max_length=200, blank=True, null=True)
    task = models.CharField(max_length=200)
    task_args = models.TextField(null=True, blank=True, verbose_name='Positional arguments')
    content = models.TextField(null=True, blank=True, verbose_name='Keyword arguments')

    active = models.BooleanField(default=True)

    task_name = models.CharField(max_length=200, unique=True)

    def get_absolute_url(self):
        return reverse('edit-scheduled-task', args=[self.pk])

    @property
    def interval_display(self):
        return 'Every %i %s' % (self.interval_count, self.interval_type if self.interval_count > 1 else
            self.interval_type[:-1])

    @property
    def multiplier(self):
        if self.interval_type == 'minutes':
            return 60

        if self.interval_type == 'hours':
            return 60 * 60

        if self.interval_type == 'days':
            return 86400

        return 1

    @property
    def positional_arguments(self):
        if self.task_args:
            return tuple([a.strip() for a in self.task_args.split(',') if a])
        else:
            return ()

    def publish(self, priority=0):
        from carrot.utilities import publish_message
        kwargs = json.loads(self.content or '{}')
        if isinstance(kwargs, str):
            kwargs = {}
        return publish_message(self.task, *self.positional_arguments, priority=priority, queue=self.queue,
                               exchange=self.exchange or '', routing_key=self.routing_key or self.queue,
                               **kwargs)

    def __str__(self):
        return self.task





# Generated by Django 2.0.4 on 2018-05-30 12:19

from django.db import migrations


def add_task_name(apps, schema_editor):
    """
    This function loops through all ScheduledTask objects and sets the task_name to a unique value, based on the
    existing task attribute, e.g.
    myapp.mytasks.hello-world
    myapp.mytasks.hello-world-1
    myapp.mytasks.hello-world-2
    """
    ScheduledTask = apps.get_model("carrot", "ScheduledTask")
    db_alias = schema_editor.connection.alias

    for index, t in ScheduledTask.objects.using(db_alias):
        task_name = t.task
        others = ScheduledTask.objects.using(db_alias).filter(task=task_name).exclude(pk=t.pk)
        if others:
            task_name = '%s-%i' % (task_name, index + 1)
        t.task_name = task_name
        t.save()


def remove_task_name(apps, schema_editor):
    """
    This doesn't need to be implemented as the task names will be deleted when the field is removed
    """
    pass


class Migration(migrations.Migration):

    dependencies = [
        ('carrot', '0002_add_task_name'),
    ]

    operations = [
        migrations.RunPython(add_task_name, remove_task_name),
    ]



# Generated by Django 2.0.4 on 2018-05-30 12:19

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('carrot', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='scheduledtask',
            name='task_name',
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
    ]



import time

from carrot.consumer import ConsumerSet, LOGGING_FORMAT
from carrot.models import ScheduledTask
from carrot.objects import VirtualHost
from carrot.scheduler import ScheduledTaskManager
from django.core.management.base import BaseCommand
from django.conf import settings
from carrot import DEFAULT_BROKER
import sys
import os
import logging
import signal
import psutil


class Command(BaseCommand):
    """
    The main process for creating and running :class:`carrot.consumer.ConsumerSet` objects and starting thes scheduler
    """
    run = True
    help = 'Starts the carrot service.'
    scheduler = None
    active_consumer_sets = []

    def __init__(self, stdout=None, stderr=None, nocolor=False):
        signal.signal(signal.SIGINT, self.exit_gracefully)
        signal.signal(signal.SIGTERM, self.exit_gracefully)
        super(Command, self).__init__(stdout, stderr, nocolor)

    def exit_gracefully(self, signum, frame):
        self.stdout.write(self.style.WARNING('Shutdown requested'))
        self.run = False

    def terminate(self, *args):
        if self.scheduler:
            self.scheduler.stop()
            self.stdout.write(self.style.SUCCESS('Successfully closed scheduler'))

        self.stdout.write('Terminating running consumer sets (%i)...' % len(self.active_consumer_sets))
        count = 0
        for consumer_set in self.active_consumer_sets:
            count += 1
            consumer_set.stop_consuming()

        self.stdout.write(self.style.SUCCESS('Successfully closed %i consumer sets' % count))
        sys.exit()

    def add_arguments(self, parser):
        parser.add_argument("-l", "--logfile", type=str, help='The path to the log file',
                            default='/var/log/carrot.log')
        parser.add_argument('--no-scheduler', dest='run_scheduler', action='store_false', default=False,
                            help='Do not start scheduled tasks (only runs consumer sets)')
        parser.set_defaults(run_scheduler=True)
        parser.set_defaults(testmode=False)
        parser.add_argument('--loglevel', type=str, default='DEBUG', help='The logging level. Must be one of DEBUG, '
                                                                          'INFO, WARNING, ERROR, CRITICAL')
        parser.add_argument('--testmode', dest='testmode', action='store_true', default=False,
                            help='Run in test mode. Prevents the command from running as a service. Should only be '
                                 'used when running Carrot\'s tests')

    def handle(self, **options):
        """
        The actual handler process. Performs the following actions:

        - Initiates and starts a new :class:`carrot.objects.ScheduledTaskManager`, which schedules all *active*
        :class:`carrot.objects.ScheduledTask` instances to run at the given intervals. This only happens if the
        **--no-scheduler** argument has not been provided - otherwise, the service only creates consumer objects

        - Loops through the queues registered in your Django project's settings module, and starts a
        new :class:`carrot.objects.ConsumerSet` for them. Each ConsumerSet will contain **n**
        :class:`carrot.objects.Consumer` objects, where **n** is the concurrency setting for the given queue (as
        defined in the Django settings)

        - Enters into an infinite loop which monitors your database for changes to your database - if any changes
        to the :class:`carrot.objects.ScheduledTask` queryset are detected, carrot updates the scheduler
        accordingly

        On receiving a **KeyboardInterrupt**, **SystemExit** or SIGTERM, the service first turns off each of the
        schedulers in turn (so no new tasks can be published to RabbitMQ), before turning off the Consumers in turn.
        The more Consumers/ScheduledTask objects you have, the longer this will take.

        :param options: provided by **argparse** (see above for the full list of available options)

        """
        signal.signal(signal.SIGTERM, self.terminate)

        # check if carrot service is already running, and warn the user if so
        running_pids = []
        for q in psutil.process_iter():
            if 'python' in q.name():
                if len(q.cmdline()) > 1 and 'manage.py' in q.cmdline()[1] and 'carrot' in q.cmdline()[2]:
                    if not q._pid == os.getpgid(0):
                        running_pids.append(q._pid)

        if running_pids:
            self.stdout.write(
                self.style.WARNING('WARNING: Carrot service is already running with the following PID. Running more '
                                   'than one instance of carrot may lead to a memory leak:\n%s'
                                   % '\n'.join([str(pid) for pid in running_pids])))

        run_scheduler = options['run_scheduler']

        try:
            queues = [q for q in settings.CARROT['queues'] if q.get('consumable', True)]

        except (AttributeError, KeyError):
            queues = [{
                'name': 'default',
                'host': DEFAULT_BROKER
            }]

        if run_scheduler:
            self.scheduler = ScheduledTaskManager()

        try:
            # scheduler
            if self.scheduler:
                self.scheduler.start()
                self.stdout.write(self.style.SUCCESS('Successfully started scheduler'))

            # logger
            loglevel = getattr(logging, options.get('loglevel', 'DEBUG'))

            logger = logging.getLogger('carrot')
            logger.setLevel(loglevel)

            file_handler = logging.FileHandler(options['logfile'])
            file_handler.setLevel(loglevel)

            stream_handler = logging.StreamHandler()
            stream_handler.setLevel(loglevel)

            formatter = logging.Formatter(LOGGING_FORMAT)
            file_handler.setFormatter(formatter)
            stream_handler.setFormatter(formatter)

            logger.addHandler(file_handler)
            logger.addHandler(stream_handler)

            # consumers
            for queue in queues:
                kwargs = {
                    'queue': queue['name'],
                    'logger': logger,
                    'concurrency': queue.get('concurrency', 1),
                }

                if queue.get('consumer_class', None):
                    kwargs['consumer_class'] = queue.get('consumer_class')

                try:
                    vhost = VirtualHost(**queue['host'])
                except TypeError:
                    vhost = VirtualHost(url=queue['host'])

                c = ConsumerSet(host=vhost, **kwargs)
                c.start_consuming()
                self.active_consumer_sets.append(c)
                self.stdout.write(self.style.SUCCESS('Successfully started %i consumers for queue %s'
                                                     % (c.concurrency, queue['name'])))

            self.stdout.write(self.style.SUCCESS('All queues consumer sets started successfully. Full logs are at %s.'
                                                 % options['logfile']))

            qs = ScheduledTask.objects.filter(active=True)
            self.pks = [t.pk for t in qs]

            while True:
                time.sleep(1)
                if not self.run:
                    self.terminate()

                if self.scheduler or options['testmode']:
                    new_qs = ScheduledTask.objects.filter(active=True)

                    if new_qs.count() > len(self.pks):
                        print('New active scheduled tasks have been added to the queryset')
                        new_tasks = new_qs.exclude(pk__in=self.pks) or [ScheduledTask()]
                        for new_task in new_tasks:
                            print('adding new task %s' % new_task)
                            self.scheduler.add_task(new_task)

                        self.pks = [t.pk for t in new_qs]

                    elif new_qs.count() < len(self.pks):
                        self.pks = [t.pk for t in new_qs]

                if options['testmode']:
                    print('TESTMODE:', options['testmode'])
                    raise SystemExit()

        except Exception as err:
            self.stderr.write(self.style.ERROR(err))

        except (KeyboardInterrupt, SystemExit):
            # self.terminate()
            pass




"""
This module contains a number of helper functions for performing basic Carrot functions, e.g. publish, schedule and
consume

Most users should use the functions defined in this module, rather than attempting to subclass the base level objects

"""


import json
from json2html.jsonconv import Json2Html
import importlib
from django.conf import settings
from carrot.objects import VirtualHost, Message
from carrot.models import ScheduledTask
from carrot.consumer import ConsumerSet
from django.utils.decorators import method_decorator
from carrot import DEFAULT_BROKER
from carrot.exceptions import CarrotConfigException


def get_host_from_name(name):
    """
    Gets a host object from a given queue name based on the Django configuration

    If no queue name is provided (as may be the case from some callers), this function returns a VirtualHost based on
    the CARROT.default_broker value.

    May raise an exception if the given queue name is not registered in the settings.

    :param str name: the name of the queue to lookup.
    :rtype: :class:`carrot.objects.VirtualHost`

    """

    try:
        if not name:
            try:
                conf = settings.CARROT.get('default_broker', {})
            except AttributeError:
                conf = {}

            if not conf:
                conf = {'url': DEFAULT_BROKER}
            elif isinstance(conf, str):
                conf = {'url': conf}

            return VirtualHost(**conf)

        queues = settings.CARROT.get('queues', [])
        queue_host = list(filter(lambda queue: queue['name'] == name, queues))[0]['host']
        try:
            vhost = VirtualHost(**queue_host)
        except TypeError:
            vhost = VirtualHost(url=queue_host)

        return vhost

    except IndexError:
        raise CarrotConfigException('Cannot find queue called %s in settings.CARROT queue list' % name)


def validate_task(task):
    """
    Helper function for dealing with task inputs which may either be a callable, or a path to a callable as a string

    In case of a string being provided, this function checks whether the import path leads to a valid callable

    Otherwise, the callable is converted back into a string (as the :class:`carrot.objects.Message` requires a string
    input)

    This function is used by the following other utility functions:
    - :func:`.create_scheduled_task`
    - :func:`.create_message`

    :param task: a callable or a path to one as a string
    :type task: str or callable
    :return: a validated path to the callable, as a string

    """
    mod, fname = (None, ) * 2

    if isinstance(task, str):
        try:
            fname = task.split('.')[-1]
            mod = '.'.join(task.split('.')[:-1])
            module = importlib.import_module(mod)
            getattr(module, fname)
        except ImportError as err:
            raise ImportError('Unable to find the module: %s' % err)

        except AttributeError as err:
            raise AttributeError('Unable to find a function called %s in the module %s: %s' % (fname, mod, err))
    else:
        task = '%s.%s' % (task.__module__, task.__name__)

    return task


def create_message(task, priority=0, task_args=(), queue=None, exchange='', routing_key=None, task_kwargs=None):
    """
    Creates a :class:`carrot.objects.Message` object without publishing it

    The task to execute (as a string or a callable) needs to be supplied. All other arguments are optional
    :param task: the task to be handled asynchronously.
    :type task: str or func
    :param int priority: the priority to be applied to the message when it is published to RabbitMQ

    :param tuple task_args: the positional arguments to be passed to the function when it is called

    :param queue: the name of the queue to publish the message to. Will be set to "default" if not provided
    :type queue: str or None
    :param str exchange: the exchange name
    :param routing_key: the routing key
    :type routing_key: str or NoneType

    :param dict task_kwargs: the keyword arguments to be passed to the function when it is called
    :rtype: :class:`carrot.objects.Message`

    """

    if not task_kwargs:
        task_kwargs = {}

    task = validate_task(task)

    vhost = get_host_from_name(queue)
    msg = Message(virtual_host=vhost, queue=queue, routing_key=routing_key, exchange=exchange, task=task,
                  priority=priority, task_args=task_args, task_kwargs=task_kwargs)

    return msg


def publish_message(task, *task_args, priority=0, queue=None, exchange='', routing_key=None, **task_kwargs):
    """
    Wrapped for :func:`.create_message`, which publishes the task to the queue

    This function is the primary method of publishing tasks to a message queue
    """
    msg = create_message(task, priority, task_args, queue, exchange, routing_key, task_kwargs)
    return msg.publish()


def create_scheduled_task(task, interval, queue=None, **kwargs):
    """
    Helper function for creating a :class:`carrot.models.ScheduledTask`

    :param task: a callable, or a valid path to one as a string
    :type task: str or callable
    :param dict interval: the interval at which to publish the message, as a dict, e.g.: {'seconds': 5}
    :param str queue: the name of the queue to publish the message to.
    :param kwargs: the keyword arguments to be passed to the function when it is executed
    :rtype: :class:`carrot.models.ScheduledTask`

    """

    task = validate_task(task)

    try:
        assert isinstance(interval, dict)
        assert len(interval.items()) == 1
    except AssertionError:
        raise AttributeError('Interval must be a dict with a single key value pairing, e.g.: {\'seconds\': 5}')

    type, count = list(*interval.items())

    t = ScheduledTask.objects.create(
        queue=queue,
        interval_type=type,
        interval_count=count,
        routing_key=queue,
        task=task,
        content=json.dumps(kwargs or '{}'),
    )
    return t


def get_mixin(decorator):
    """
    Helper function that allows dynamic application of decorators to a class-based views

    :param func decorator: the decorator to apply to the view
    :return:
    """
    class Mixin:
        @method_decorator(decorator)
        def dispatch(self, request, *args, **kwargs):
            return super(Mixin, self).dispatch(request, *args, **kwargs)

    return Mixin


def create_class_view(view, decorator):
    """
    Applies a decorator to the dispatch method of a given class based view. Can be chained

    :param class view: the class-based view to apply the decorator to
    :param function decorator: the decorator to apply

    :rtype: the updated class based view
    """
    class DecoratedView(get_mixin(decorator), view):
        pass

    return DecoratedView


def decorate_class_view(view_class, decorators=None):
    """
    Loop through a list of string paths to decorator functions, and call :func:`.create_class_view` for each one

    :param class view_class: the class-based view to attach the decorators to
    :param list decorators: a list of string decorators, e.g. ['myapp.mymodule.decorator1', 'myapp.mymodule.decorator2']

    :return: the class based view with all decorators attached to the dispatch method

    """
    if decorators is None:
        decorators = []

    for decorator in decorators:
        _module = '.'.join(decorator.split('.')[:-1])
        module = importlib.import_module(_module)
        _decorator = getattr(module, decorator.split('.')[-1])
        view_class = create_class_view(view_class, _decorator)

    return view_class


def create_function_view(view, decorator):
    """
    Similar to :func:`.create_class_view`, but attaches a decorator to a function based view, instead of a class-based
    one

    :param func view: the function based view to attach a decorator tio
    :param func decorator: the decorator attach

    :rtype: the updated view function

    """
    @decorator
    def wrap(request, *args, **kwargs):
        return view(request, *args, **kwargs)

    return wrap


def decorate_function_view(view, decorators=None):
    """
    Similar to :func:`.decorate_class_view`, but for function based views
    """
    if not decorators:
        decorators = []

    for decorator in decorators:
        _module = '.'.join(decorator.split('.')[:-1])
        module = importlib.import_module(_module)
        _decorator = getattr(module, decorator.split('.')[-1])
        view = create_function_view(view, _decorator)

    return view



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# carrot documentation build configuration file, created by
# sphinx-quickstart on Tue Sep 26 15:41:05 2017.
#

import sys
import os
import sphinx_bootstrap_theme
import django
from carrot import __version__


BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

sys.path.insert(0, os.path.abspath('.'))
sys.path.append(BASE_DIR)
sys.path.append(os.path.dirname(BASE_DIR))

os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'doc_settings')
django.setup()

extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.doctest',
    'sphinx.ext.intersphinx',
    'sphinx.ext.coverage',
]

templates_path = ['_templates']

source_suffix = '.rst'

master_doc = 'index'

# General information about the project.
project = 'django-carrot'
copyright = '2017-2018, Christopher Davies'
author = 'Christopher Davies'

version = __version__
release = __version__

language = None

exclude_patterns = []

pygments_style = 'sphinx'

todo_include_todos = False

html_theme = 'bootstrap'
html_theme_path = sphinx_bootstrap_theme.get_html_theme_path()

html_theme_options = {
    # Tab name for entire site. (Default: "Site")
    'navbar_site_name': "Topics",

    'navbar_links': [
        ("Home", "index"),
        ("What's new", "release-notes"),
        ("Getting started", "quick-start"),
        ("Monitor", "monitor"),
        ("Configuration", "settings"),
        ("API", "api"),

    ],

    # Render the next and previous page links in navbar. (Default: true)
    'navbar_sidebarrel': False,

    'globaltoc_depth': -1,

    # Render the current pages TOC in the navbar. (Default: true)
    'navbar_pagenav': False,

    # Location of link to source.
    'source_link_position': None,

    # - Bootstrap 3: https://bootswatch.com/3
    'bootswatch_theme': "united",
}

html_static_path = ['_static']

htmlhelp_basename = 'carrotdoc'


latex_elements = {}

latex_documents = [
    (master_doc, 'carrot.tex', 'carrot Documentation',
     'Christopher Davies', 'manual'),
]

man_pages = [
    (master_doc, 'carrot', 'carrot Documentation',
     [author], 1)
]

texinfo_documents = [
    (master_doc, 'carrot', 'carrot Documentation',
     author, 'carrot', 'One line description of project.',
     'Miscellaneous'),
]

intersphinx_mapping = {'https://docs.python.org/': None}


def setup(app):
    app.add_stylesheet('carrot.css')




# Generated by Django 2.0.5 on 2018-05-08 12:04

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='MessageLog',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', models.CharField(choices=[('PUBLISHED', 'Published'), ('IN_PROGRESS', 'In progress'), ('FAILED', 'Failed'), ('COMPLETED', 'Completed')], default='PUBLISHED', max_length=11)),
                ('exchange', models.CharField(blank=True, max_length=200, null=True)),
                ('queue', models.CharField(blank=True, max_length=200, null=True)),
                ('routing_key', models.CharField(blank=True, max_length=200, null=True)),
                ('uuid', models.CharField(max_length=200)),
                ('priority', models.PositiveIntegerField(default=0)),
                ('task', models.CharField(max_length=200)),
                ('task_args', models.TextField(blank=True, null=True, verbose_name='Task positional arguments')),
                ('content', models.TextField(blank=True, null=True, verbose_name='Task keyword arguments')),
                ('exception', models.TextField(blank=True, null=True)),
                ('traceback', models.TextField(blank=True, null=True)),
                ('output', models.TextField(blank=True, null=True)),
                ('publish_time', models.DateTimeField(blank=True, null=True)),
                ('failure_time', models.DateTimeField(blank=True, null=True)),
                ('completion_time', models.DateTimeField(blank=True, null=True)),
                ('log', models.TextField(blank=True, null=True)),
            ],
            options={
                'ordering': ('-failure_time', '-completion_time', 'status', '-priority', '-publish_time'),
            },
        ),
        migrations.CreateModel(
            name='ScheduledTask',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('interval_type', models.CharField(choices=[('seconds', 'seconds'), ('minutes', 'minutes'), ('hours', 'hours'), ('days', 'days')], default='seconds', max_length=200)),
                ('interval_count', models.PositiveIntegerField(default=1, validators=[django.core.validators.MinValueValidator(1)])),
                ('exchange', models.CharField(blank=True, max_length=200, null=True)),
                ('routing_key', models.CharField(blank=True, max_length=200, null=True)),
                ('queue', models.CharField(blank=True, max_length=200, null=True)),
                ('task', models.CharField(max_length=200)),
                ('task_args', models.TextField(blank=True, null=True, verbose_name='Positional arguments')),
                ('content', models.TextField(blank=True, null=True, verbose_name='Keyword arguments')),
                ('active', models.BooleanField(default=True)),
            ],
        ),
    ]
